(* structure Main: The main program. 
   Builds and connects lexers and parsers,
   implements a simple top-level evaluation loop (Main.main).
   
   For most experiments with the implementation, it should not be
   necessary to change this file.
*)

structure Main =
struct

open PPUtil
open Syntax
open Core

(* 
 * We apply the functors generated from tyarith.lex and tyarith.grm to produce
 * the TyArithParser structure.
 *)

  structure TyArithLrVals =
    TyArithLrValsFun(structure Token = LrParser.Token)

  structure TyArithLex =
    TyArithLexFun(structure Tokens = TyArithLrVals.Tokens)

  structure TyArithParser =
    Join(structure LrParser = LrParser
	 structure ParserData = TyArithLrVals.ParserData
	 structure Lex = TyArithLex)

(* 
 * We need a function which given a lexer invokes the parser. The
 * function invoke does this.
 *)

  fun invoke lexstream =
      let fun print_error (s,i:int,_) =
	      TextIO.output(TextIO.stdOut,
			    "Error, line " ^ (Int.toString i) ^ ", " ^ s ^ "\n")
       in TyArithParser.parse(0,lexstream,print_error,())
      end

(* 
 * Finally, we need a driver function that reads one or more expressions
 * from the standard input. The function main, shown below, does
 * this. It runs the evaluator on the standard input and terminates when
 * an end-of-file is encountered.
 *)

  (* dummy tokens *)
  val dummyEOF = TyArithLrVals.Tokens.EOF(0,0)
  val dummySEMI = TyArithLrVals.Tokens.SEMI(0,0)

  fun main () = 
      let val lexer = TyArithParser.makeLexer (fn _ => TextIO.inputLine TextIO.stdIn)
	  fun loop lexer =
	      let val (result,lexer) = invoke lexer
		  val (nextToken,lexer) = TyArithParser.Stream.get lexer
		  val _ = case result
			    of SOME t =>
			        (print_string "value: "; printtm(eval t);
				 print_break 1 2; print_string ": ";
				 printty (typeof t);
				 print_newline())
			     | NONE => (print_string "nothing"; print_newline())
	       in if TyArithParser.sameToken(nextToken,dummyEOF) then ()
		  else loop lexer
	      end
       in loop lexer
      end

end (* structure Main *)
